{*******************************************************************************
* Unit      : TSLockPK Demostration program "LockTest"                         *
*             Uses SLockPK in the "normal" or "Challenge-response"             *
*             registration mode, and uses the Clipboard as the medium for      *
*             submitting Certificates to SLockPK                               *
********************************************************************************
* Purpose   : Demonstration/Example program for the SLockPK component          *
*             This program is intended to provide SLockPK users with ideas     *
*             and examples how to use the SLockPK component to achieve         *
*             successful results. You are at liberty to use sections of this   *
*             example code in your applications when you have purchased a      *
*             license for SLockPK                                              *
*                                                                              *
*             This example program also demonstrates how to use Reset and      *
*             Termaination Certificates.                                       *
********************************************************************************
* Copyright : This software  is copyright TSM Inc. 1998-1999                   *
*             This source code may not be distributed to third parties in      *
*             or in part without the written permission of TSM Inc.            *
*             All rights reserved. Liability limited to replacement of         *
*             this original source code in the case of loss or damage because  *
*             the use or misuse of this software.                              *
********************************************************************************
* Version   : 14.05.98  - 1.00  Original Version                               *
*             22.04.99  - 2.00  PK version "SLockPK"                           *
********************************************************************************
* Switches  : None                                                             *
*******************************************************************************}
unit Slockfrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, registry, RegFrm, Slock, ExtCtrls, Menus;

type
  TfrmMain = class(TForm)
    btnRegister: TButton;
    grpStatus: TGroupBox;
    lblStatus: TLabel;
    lblStatusDisp: TLabel;
    lblDaysLeft: TLabel;
    lblDaysLeftDisp: TLabel;
    lblStartsLeftDisp: TLabel;
    lblExpiryDateDisp: TLabel;
    lblProtType: TLabel;
    lblProtTypeDisp: TLabel;
    lblStartsLeft: TLabel;
    lblExpiryDate: TLabel;
    lblRegisteredTo: TLabel;
    lblRegisteredToDisp: TLabel;
    lblChallengeString: TLabel;
    lblChallengeStringDisp: TLabel;
    bvlStatus: TBevel;
    lblGraceStartsLeft: TLabel;
    lblGraceStartsLeftDisp: TLabel;
    mnuMainMenu: TMainMenu;
    mnmFile: TMenuItem;
    mnpExit: TMenuItem;
    mnmRegister: TMenuItem;
    mnpRegNow: TMenuItem;
    mnmFunction: TMenuItem;
    mnpNormFunc: TMenuItem;
    btnExit: TButton;
    lblVersionDisp: TLabel;
    Slock1: TSlock;
    N1: TMenuItem;
    mnpLoadCertificate: TMenuItem;
    lblCertExpireDisp: TLabel;
    lblCertExpire: TLabel;
    lblVersionLabel: TLabel;
    lblSerialNumber: TLabel;
    lblSerialNumberDisp: TLabel;
    lblUser1Label: TLabel;
    lblUser2Label: TLabel;
    lblUser1: TLabel;
    lblUser2: TLabel;
    procedure Slock1FirstStart(Sender: TObject);
    procedure Slock1ClockMovedBack(Sender: TObject);
    procedure Slock1Reminder(Sender: TObject);
    procedure Slock1WrongUnlockCode(Sender: TObject);
    procedure Slock1Expired(Sender: TObject);
    procedure btnRegisterClick(Sender: TObject);
    procedure Slock1Register(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Slock1RegInfoChanged(Sender: TObject);
    procedure Slock1InGracePeriod(Sender: TObject);
    procedure Slock1RegWriteErr(Sender: TObject);
    procedure mnpNormFuncClick(Sender: TObject);
    procedure mnpExitClick(Sender: TObject);
    procedure mnpRegNowClick(Sender: TObject);
    procedure btnExitClick(Sender: TObject);
    procedure Slock1Extend(Sender: TObject);
    procedure Slock1TimedOut(Sender: TObject);
    procedure Slock1RegInfoTamper(Sender: TObject);
    procedure Slock1NoInitCert(Sender: TObject);
    procedure Slock1CertAccept(Sender: TObject);
    procedure Slock1CertExpired(Sender: TObject);
    procedure Slock1CertReject(Sender: TObject);
    procedure Slock1TrialInfoTamper(Sender: TObject);
    procedure Slock1Reset(Sender: TObject);
    procedure mnpLoadCertificateClick(Sender: TObject);
    procedure Slock1Terminate(Sender: TObject);
  private
    procedure UpdateStatus;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

{*******************************************************************************
* Procedure : btnRegisterClick                                                 *
********************************************************************************
* Purpose   : Opens up the window in which the registration information can    *
*             be entered into                                                  *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.btnRegisterClick(Sender: TObject);
var
     frmRegister: TfrmRegister;
begin
     // create the registration window
     frmRegister := TfrmRegister.Create(self);

     // set the caption and show the window
     frmRegister.Caption := 'Register ' + Caption;
     frmRegister.ShowModal;

     // release the window
     frmRegister.Free;

     // Update the status
     UpdateStatus;

end; {btnRegisterClick}

{*******************************************************************************
* Procedure : FormCreate                                                       *
********************************************************************************
* Purpose   : We check the current status of the protection in the form create *
*             of the main form. (Slock1.CheckProtection) The internal state of *
*             Slock is set, and we then use 'UpdateStatus' to read the status  *
*             of Slock out into the status display                             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.FormCreate(Sender: TObject);
var
     TempString: string;
begin
     // load the secret information into the SLock component as encrypted data.
     // This stops disassemblers reading the information from
     // the delphi form properties. The Info loaded by the LoadInfoString method
     // has to be encrypted using the SLockPK certificate builder
     // you should build the Public key string dynamically so that attackers
     // cannot just swap their key for your key
     // for higher security methods, see the LockPK application which implemets
     // many more features

     // build the public key - do this in two parts and swap them to make
     // patching in new keys more difficult. For higher security methods see
     // the LockPK application
     TempString := 'Am8PgXDQINe0vNOzPW2PB0rvIeC+tURiCfqGkVViRohk+DhU';
     TempString := '++11Ik:wowtWIExeqGaN9Uyzq5LdTsLcSnCOPSThLUcS7' + TempString;

     // load the PK key into SLockPK
     SLock1.LoadPKKey(TempString);

     // Show the version of the SLock component we are using
     lblVersionDisp.Caption := SLock1.GetVersion;

     // check the protection of the Slock component
     SLock1.CheckProtection;

     // Update the status display and the state of the Buttons/Menupoints
     UpdateStatus;
end; {FormCreate}

{*******************************************************************************
* Procedure : UpdateStatus                                                     *
********************************************************************************
* Purpose   : Loads the internal status of the Slock component into the status *
*             display. Also sets the state of the menus and buttons            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.UpdateStatus;
begin
     // load the status display from the Status property
     case Slock1.Status of
     stNotExpired:
          lblStatusDisp.Caption := 'Not Expired';
     stExpired:
          lblStatusDisp.Caption := 'Expired';
     stGrace:
          lblStatusDisp.Caption := 'Grace Period';
     stRegistered:
          lblStatusDisp.Caption := 'Registered';
     stDemo:
          lblStatusDisp.Caption := 'Registered Demo';
     end;

     // show the certificate expiry date
     lblCertExpire.Caption := FormatDateTime('dd/mm/yyyy', SLock1.CertExpire);

     // load the days/starts left display
     case Slock1.ProtectionType of
     ptStartCount:
          begin
               lblProtTypeDisp.Caption := 'Start Count';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := IntToStr(Slock1.TrialPeriod);
               lblGraceStartsLeftDisp.Caption := IntToStr(Slock1.GracePeriod);
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     ptDayCount:
          begin
               lblProtTypeDisp.Caption := 'Day Count';
               lblDaysLeftDisp.Caption := IntToStr(Slock1.TrialPeriod);
               lblStartsLeftDisp.Caption := 'N/A';
               lblGraceStartsLeftDisp.Caption := IntToStr(Slock1.GracePeriod);
               lblExpiryDateDisp.Caption := DateToStr(Slock1.ExpiryDate);
          end;
     ptExpiryDate:
          begin
               lblProtTypeDisp.Caption := 'Expiry Date';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := 'N/A';
               lblGraceStartsLeftDisp.Caption := IntToStr(Slock1.GracePeriod);
               lblExpiryDateDisp.Caption := DateToStr(Slock1.ExpiryDate);
          end;
     ptTimed:
          begin
               lblProtTypeDisp.Caption := 'Timed';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := 'N/A';
               lblGraceStartsLeftDisp.Caption := 'N/A';
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     ptDaysUsed:
          begin
               lblProtTypeDisp.Caption := 'Days Used';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := IntToStr(Slock1.TrialPeriod);
               lblGraceStartsLeftDisp.Caption := IntToStr(Slock1.GracePeriod);
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     ptNone:
          begin
               lblProtTypeDisp.Caption := 'None';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := 'N/A';;
               lblGraceStartsLeftDisp.Caption := 'N/A';;
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     ptPK:
          begin
               lblProtTypeDisp.Caption := 'PK Override';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := 'N/A';;
               lblGraceStartsLeftDisp.Caption := 'N/A';;
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     end;

     // load the registration information
     if SLock1.Registered then
     begin
          lblRegisteredToDisp.Caption := Slock1.RegName;
     end
     else
     begin
          lblRegisteredToDisp.Caption := 'N/A';
     end;

     // the serial number is a persistent field which can be set in trial mode
     lblSerialNumberDisp.Caption := Slock1.SerialNumber;

     // the user fields are persistent fields
     lblUser1.Caption := SLock1.GetUserField1;
     lblUser2.Caption := SLock1.GetUserField2;

     {set the state of the buttons/menus - these are set for this demo
     application and probably won't be the same same as for a real-life
     application}
     // the demonstration function should work until the expiry is reached
     mnmFunction.Enabled := (Slock1.Status <> stExpired);
end; {UpdateStatus}

{*******************************************************************************
* Procedure : mnpNormFuncClick                                                 *
********************************************************************************
* Purpose   : This is an example of using Slock to control access to functions *
*             in the 'host' program. The menu point 'Function' is enabled      *
*             or disabled according to the status of Slock.Status.             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpNormFuncClick(Sender: TObject);
begin
     ShowMessage('This is a normal function - it will be disabled after the program expired.');
end; {mnpNormFuncClick}

{*******************************************************************************
* Procedure : mnpRegNowClick                                                   *
********************************************************************************
* Purpose   : This calls event 'btnRegisterClick' to register Slock            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpRegNowClick(Sender: TObject);
begin
     btnRegisterClick(self);
end; {mnpRegNowClick}

{*******************************************************************************
* Procedure : Slock1FirstStart                                                 *
********************************************************************************
* Purpose   : This is called the first time that the Slock.CheckProtection     *
*             Method is called (for the current registry key settings). You    *
*             can use this to display a welcome message or Licensing terms and *
*             conditions or even an advert for your other software.            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1FirstStart(Sender: TObject);
begin
     ShowMessage('This is the first start! Welcome to the Slock demonstration program by TSM Inc.');

     UpdateStatus;
end; {Slock1FirstStart}

{*******************************************************************************
* Procedure : btnExitClick                                                     *
********************************************************************************
* Purpose   : Closes the program.                                              *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.btnExitClick(Sender: TObject);
begin
     Close;
end; {btnExitClick}

{*******************************************************************************
* Procedure : mnpExitClick                                                     *
********************************************************************************
* Purpose   : Closes the program.                                              *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpExitClick(Sender: TObject);
begin
     Close;
end; {mnpExitClick}

{*******************************************************************************
* Procedure : TfrmMain.mnpReadCertificateClick                                 *
********************************************************************************
* Purpose   : Allows you to read a Certificate into the example application    *
*             manually. Useful for reading in Reset Certificates.              *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpLoadCertificateClick(Sender: TObject);
begin
     ShowMessage('Place the Certificate on the Windows Clipboard and press OK.');
     SLock1.LoadCertClipboard;
end; {TfrmMain.mnpReadCertificateClick}

{*******************************************************************************
* Procedure : TfrmMain.Slock1CertAccept                                        *
********************************************************************************
* Purpose   : This event is triggered when SLockPK finds a valid Certificate   *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1CertAccept(Sender: TObject);
begin
     // the certificate was OK
     ShowMessage('The certificate was accepted.');

     UpdateStatus;
end; {TfrmMain.Slock1CertAccept}

{*******************************************************************************
* Procedure : TfrmMain.Slock1CertExpired                                       *
********************************************************************************
* Purpose   : This event is triggered when the SLockPK Certificate has expired *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1CertExpired(Sender: TObject);
begin
     ShowMessage('The existing certificate has expired. Copy a new certificate ' +
                 'to the clipboard and press OK to accept the certificate into ' +
                 'Locktest.');

     // try to read the clipboard for a certificate
     SLock1.LoadCertClipboard;

     UpdateStatus;
end; {TfrmMain.Slock1CertExpired}

{*******************************************************************************
* Procedure : TfrmMain.Slock1CertReject                                        *
********************************************************************************
* Purpose   : This event is triggered when the SLockPK Certificate supplied    *
*             is not valid                                                     *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1CertReject(Sender: TObject);
begin
     if MessageDlg ('The certificate was rejected or no certificate was found. ' +
                    'To submit a new Certificate to SLockPK, paste it onto the ' +
                    'clipboard now. Do you want to attempt to load the Certificate?',
                    mtConfirmation,[mbYes, mbNo], 0) = mrYes then
     begin
          // try to read the clipboard for a certificate
          SLock1.LoadCertClipboard;

          UpdateStatus;
     end
     else
     begin
          // Stop as we do not have a Certificate
          ShowMessage('LockTest cannot run without a Certificate and will now end.');
          Halt;
     end;
end; {TfrmMain.Slock1CertReject}

{*******************************************************************************
* Procedure : Slock1ClockMovedBack                                             *
********************************************************************************
* Purpose   : People often try to get around a fixed expiry date by setting    *
*             the clock back. This event is triggered when the system clock    *
*             been set back to a date that is before the Slock was even        *
*             created. Normally you would end the program under such a         *
*             a condition.                                                     *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1ClockMovedBack(Sender: TObject);
begin
     ShowMessage('The clock was moved back!');

     UpdateStatus;
end; {Slock1ClockMovedBack}

{*******************************************************************************
* Procedure : Slock1Expired                                                    *
********************************************************************************
* Purpose   : This event is triggered when the 'CheckProtection' method is     *
*             called and the component has expired (and there is no grace      *
*             period left).                                                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Expired(Sender: TObject);
begin
     ShowMessage('The program has expired. Some Functions have been disabled. Register to re-enable them.');

     UpdateStatus;
end; {Slock1Expired}

{*******************************************************************************
* Procedure : Slock1Extend                                                     *
********************************************************************************
* Purpose   : This event is triggered when the 'RegisterNow' method is called  *
*             with an extension code. The Extension code increments the        *
*             grace period.                                                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Extend(Sender: TObject);
begin
     ShowMessage('The trial period has been extended!');

     UpdateStatus;
end; {Slock1Extend}

{*******************************************************************************
* Procedure : Slock1InGracePeriod                                              *
********************************************************************************
* Purpose   : This is triggered when the program has expired, but is still in  *
*             the grace period. This gives you the opportunity to display a    *
*             more urgent message to promt the user that his software will soon*
*             expire.                                                          *
*             The procedure shown here is more complcated than normally,       *
*             because we want to be able to demonstrate all the different forms*
*             of protection.                                                   *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1InGracePeriod(Sender: TObject);
begin
     if Slock1.ProtectionType = ptStartCount then
          ShowMessage('This is the GRACE PERIOD: ' + IntToStr(Slock1.GracePeriod) +
                      ' starts left in the evaluation period.');
     if Slock1.ProtectionType = ptDayCount then
          ShowMessage('This is the GRACE PERIOD: ' + IntToStr(Slock1.GracePeriod)
                       + ' days left in the evaluation period.');
     if Slock1.ProtectionType = ptExpiryDate then
          ShowMessage('This is the GRACE PERIOD: ' + IntToStr(Slock1.GracePeriod)
                       + ' days left in the evaluation period.');
     if Slock1.ProtectionType = ptDaysUsed then
          ShowMessage('This is the GRACE PERIOD: ' + IntToStr(Slock1.GracePeriod) +
                      ' starts left in the evaluation period.');

     UpdateStatus;
end; {Slock1InGracePeriod}

{*******************************************************************************
* Procedure : Slock1NoInitCert                                                 *
********************************************************************************
* Purpose   : This event is triggered when SLockPK does not find a certificate *
*             in the registry. The user should place a certificate on the      *
*             clipboard and submit this to SLOckPK.                            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1NoInitCert(Sender: TObject);
begin
     // this is a simplified procedure for accepting certificates. For the more
     // advanced version, see the "LockPK" application.
     ShowMessage('The initial certificate was not found. Copy a certificate to ' +
                 'the clipboard and press OK to accept the certificate into' +
                 'LockTest.');

     // try to read the clipboard for a certificate
     SLock1.LoadCertClipboard;

     UpdateStatus;
end; {TfrmMain.Slock1NoInitCert}

{*******************************************************************************
* Procedure : Slock1RegInfoTamper                                              *
********************************************************************************
* Purpose   : This is triggered if the registry has been tampered with (e.g.   *
*             one of the two copies of the registry information has been       *
*             changed or deleted. If this is the case, the user will have to   *
*             enter his Registration code again.                               *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1RegInfoTamper(Sender: TObject);
begin
     // we can use a slightly vague message here, as this can never normally
     // happen - we are using the Challenge-Response registration scheme and
     // therefore this cannot happen (can normally only happen if the machine
     // fingerprint information is not valid
     ShowMessage('The registration information has been tampered with.');

     UpdateStatus;
end; {Slock1RegInfoTamper}

{*******************************************************************************
* Procedure : Slock1Register                                                   *
********************************************************************************
* Purpose   : Is called when the OnRegister event in Slock is triggered. In    *
*             this example we just show a short message, but we could also     *
*             use this event to show a thankyou screen, or even enable some    *
*             additional functions.                                            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Register(Sender: TObject);
begin
     ShowMessage('The program has been registered. Thank you for registering!');

     UpdateStatus;
end; {Slock1Register}

{*******************************************************************************
* Procedure : Slock1RegWriteErr                                                *
********************************************************************************
* Purpose   : Is called when the read after write check of the Windows registry*
*             shows that the information was not correctly written.            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1RegWriteErr(Sender: TObject);
begin
     ShowMessage('There was an error writing to the registry. The program will terminate.');
     Halt;
end; {Slock1RegWriteErr}

{*******************************************************************************
* Procedure : Slock1Reminder                                                   *
********************************************************************************
* Purpose   : This is the normal reminder event which is triggered when Slock  *
*             has not been registered and has not expired. You can use this as *
*             a reminder that the user should register.                        *
*             The procedure shown here is more complcated than normally,       *
*             because we want to be able to demonstrate all the different forms*
*             of protection.                                                   *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Reminder(Sender: TObject);
begin
     if Slock1.ProtectionType = ptStartCount then
          ShowMessage('This is the normal reminder: ' + IntToStr(Slock1.TrialPeriod) +
                      ' starts left in the evaluation period.');
     if Slock1.ProtectionType = ptDayCount then
          ShowMessage('This is the normal reminder: ' + IntToStr(Trunc(Slock1.ExpiryDate - Now)) +
                      ' days left in the evaluation period.');
     if Slock1.ProtectionType = ptExpiryDate then
          ShowMessage('This is the normal reminder: ' + IntToStr(Trunc(Slock1.ExpiryDate - Now)) +
                      ' days left in the evaluation period.');
     if Slock1.ProtectionType = ptDaysUsed then
          ShowMessage('This is the normal reminder: ' + IntToStr(Slock1.TrialPeriod) +
                      ' days left in the evaluation period.');

     UpdateStatus;
end; {Slock1Reminder}

{*******************************************************************************
* Procedure : TfrmMain.Slock1Reset                                             *
********************************************************************************
* Purpose   : Resets the trial period (Caution!! - this should only be used    *
*             in exceptional circumstances - the user could keep reapplying    *
*             this certificate, so you should not really use this as a matter  *
*             of course, rather only when the application needs to be reset    *
*             for debugging or emergency purposes)                             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Reset(Sender: TObject);
begin
     ShowMessage('The trial period has been reset. You now need to submit a ' +
                 'new valid Trial Certificate to continue.');
     Halt;
end; {TfrmMain.Slock1Reset}

{*******************************************************************************
* Procedure : SLock1TimedOut                                                   *
********************************************************************************
* Purpose   : Is triggered when SLock times out (!! Timer mode only !!)        *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1TimedOut(Sender: TObject);
begin
     UpdateStatus;
     ShowMessage('The component has timed out!');
end; {Slock1TimedOut}

{*******************************************************************************
* Procedure : TfrmMain.Slock1TrialInfoTamper                                   *
********************************************************************************
* Purpose   : This event is triggered when the SLockPK unlock code does not    *
*             match the PC.                                                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1TrialInfoTamper(Sender: TObject);
begin
     ShowMessage('It appears that either you have changed some hardware settings ' +
                 'or that the software is not registered to run on this PC.' +
                 'Please contact your supplier for further information.');
end; {TfrmMain.Slock1TrialInfoTamper}

{*******************************************************************************
* Procedure : Slock1WrongUnlockCode                                            *
********************************************************************************
* Purpose   : This event is triggered when the user enters the wrong unlock    *
*             code. To discourage random attempts at breaking the code with    *
*             brute force, we end the program when the user enters the wrong   *
*             code.                                                            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1WrongUnlockCode(Sender: TObject);
begin
     ShowMessage('The unlock code was wrong. Program was not registered and will end.');
     Halt;
end; {Slock1WrongUnlockCode}

{*******************************************************************************
* Procedure : Slock1RegInfoChanged                                             *
********************************************************************************
* Purpose   : This is triggered when the registration information stored in    *
*             Windows registry is checked, and it is found that the name or    *
*             machine has been changed. The input of a new unlock string is    *
*             necessary before the component will be marked as 'registered'    *
*             again.                                                           *
*             For the machine specific registration to be active, the property *
*             'SpecificMachine' must be set to true. Use this with care, as it *
*             can lead to 'false positives' if the user changes the Windows    *
*             registration information.                                        *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1RegInfoChanged(Sender: TObject);
begin
     ShowMessage('The registration info was changed.');

     UpdateStatus;
end; {Slock1RegInfoChanged}

{*******************************************************************************
* Procedure : TfrmMain.Slock1Terminate                                         *
********************************************************************************
* Purpose   : This is triggered when a Terminate Certificate is received. The  *
*             Terminate Certificate is written to the registry and is          *
*             therefore permanent until a new Certificate is received. We have *
*             to give the user the chance to read in a new Certificate or they *
*             can never get rid of this state.                                 *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Terminate(Sender: TObject);
begin
     if MessageDlg ('LockTest has been terminated. You need to provide a new ' +
                    'valid Certificate to be able to continue. Paste the new ' +
                    'Certificate onto the Clipboard now and press "Yes" if you ' +
                    'want to try and accept it.', 
                    mtConfirmation,[mbYes, mbNo], 0) = mrYes then
     begin
          // try to read the clipboard for a certificate
          SLock1.LoadCertClipboard;

          UpdateStatus;
     end
     else
     begin
          // Stop as we do not have a Certificate
          ShowMessage('LockTest cannot run without a Certificate and will now end.');
          Halt;
     end;
end;

end.
